///////////////////////////////////////////////////////////////
//
// 21065LEzFlash.c
//
// Analog Devices, Inc. - 2005
//
// Rev - 1.00.0 - initial release
//
// VisualDSP++ "Flash Programmer" flash driver for use with the
// ADSP-21065L EZ-KIT Lite containing the STMicroelectronics 
// M29W040B flash device.
//
///////////////////////////////////////////////////////////////
#include <stdlib.h>
#include "Errors.h"
#include <def21065l.h>
#include <cdef21065l.h>
#include <signal.h>

// #defines
#define TRUE			0x1
#define FALSE			0x0
#define BUFFER_SIZE		0x600
#define NUM_SECTORS		8

#define FLASH_START 0x800000

// Flash Programmer commands
typedef enum
{
	NO_COMMAND,		// 0
	GET_CODES,		// 1
	RESET,			// 2
	WRITE,			// 3
	FILL,			// 4
	ERASE_ALL,		// 5
	ERASE_SECT,		// 6
	READ,			// 7
	GET_SECTNUM,	// 8
	GET_SECSTARTEND	// 9
}enProgCmds;

// structure for flash sector information
typedef struct _SECTORLOCATION{
	unsigned long ulStartOff;
	unsigned long ulEndOff;
}SECTORLOCATION;

// function prototypes
ERROR_CODE SetupForFlash();
ERROR_CODE GetCodes();
ERROR_CODE PollToggleBit();
ERROR_CODE ResetFlash();
ERROR_CODE EraseFlash();
ERROR_CODE EraseBlock( int nBlock );
ERROR_CODE UnlockFlash();
ERROR_CODE WriteData( unsigned long ulStart, long lCount, long lStride, int *pnData );
ERROR_CODE FillData( unsigned long ulStart, long lCount, long lStride, int *pnData );
ERROR_CODE ReadData( unsigned long ulStart, long lCount, long lStride, int *pnData );
ERROR_CODE ReadFlash(unsigned long ulOffset, int *data);
ERROR_CODE WriteFlash( unsigned long ulOffset, int *flash_data );
ERROR_CODE GetSectorNumber( unsigned long ulOffset, int *pnSector );
ERROR_CODE GetSectorStartEnd( unsigned long *ulStartOff, unsigned long *ulEndOff, int nSector );
void ExternalPort1_DMA_isr(int i);

// global data for use with the VisualDSP++ plug-in
char 			*AFP_Title 			= "ADSP-21065L EZ-KIT Lite";
char 			*AFP_Description 	= "M29W040B (512k x 8)";
char 			*AFP_DrvVersion		= "1.01.0";		// Driver Version
char			*AFP_BuildDate		= __DATE__;		// Driver Build Date
char			*AFP_DeviceCompany  = "STMicro Electronics";					// Flash Company
enProgCmds 		AFP_Command 		= NO_COMMAND;
int 			AFP_ManCode 		= -1;				// 0x20 = STMicroelectronics
int 			AFP_DevCode 		= -1;				// 0xE3 = M29W040B
unsigned long 	AFP_Offset 			= 0x0;
int 			*AFP_Buffer;
long 			AFP_Size 			= BUFFER_SIZE;
long 			AFP_Count 			= -1;
long 			AFP_Stride 			= -1;
int 			AFP_NumSectors 		= NUM_SECTORS;
long 			AFP_SectorSize 		= 0x10000;
int 			AFP_Sector 			= -1;
int 			AFP_Error 			= 0;
bool 			AFP_Verify 			= FALSE;
unsigned long 	AFP_StartOff 		= 0x0;
unsigned long 	AFP_EndOff 			= 0x0;
int				AFP_FlashWidth		= 0x10;
int 			*AFP_SectorInfo;
SECTORLOCATION 	SectorInfo[NUM_SECTORS];

// exit flag
bool bExit = FALSE;

// Global variables used for reading and writing the data from/to the external port.
int read_data = 0;
int flash_data;

main()
{
	int i = 0;
	
	// Enable the external port 1 DMA interrupt. In the ADSP-21065L EZ-KIT
	// the EPROM is connected using the BMS as CS. The WR signal is not connected
	// to the EPROM. For the incircuit programming of the flash the WR/ signal 
	// available from the J2 connector(45 pin) should be connected to the WR/
	// signal of the flash JP4 (2 pin).Since the flash is mapped to the BMS space
	// the data can be read or written using the external port 1 DMA.
	
	interrupt(SIG_EP1I,ExternalPort1_DMA_isr);

	// by making AFP_Buffer as big as possible the plug-in can send and
	// receive more data at a time making the data transfer quicker
	//
	// by allocating it on the heap the compiler does not create an
	// initialized array therefore making the driver image smaller
	// and faster to load
	//
	// we have modified the linker description file (LDF) so that the heap
	// is large enough to store BUFFER_SIZE elements at this point
	
	AFP_Buffer = (int *)malloc(BUFFER_SIZE);

	if ( AFP_Buffer == NULL )
	{
		// tell GUI that our buffer was not initialized
		AFP_Error = BUFFER_IS_NULL;
	}

	// AFP_Buffer will be NULL if we could not allocate storage for the
	// buffer
	if( AFP_Buffer != NULL )
	{
		//initiate sector information structures
		for(i=0;i<AFP_NumSectors; i++)
		{
			GetSectorStartEnd(&SectorInfo[i].ulStartOff, &SectorInfo[i].ulEndOff, i);
		}

		AFP_SectorInfo = (int*)&SectorInfo[0];

		// setup the flash so the DSP can access it
		SetupForFlash();

		// get the device id and manufacturer id
		GetCodes();

		// command processing loop
		while ( !bExit )
		{
			// the plug-in will set a breakpoint at "AFP_BreakReady" so it knows
			// when we are ready for a new command because the DSP will halt
			//
			// the jump is used so that the label will be part of the debug
			// information in the driver image otherwise it may be left out
			// since the label is not referenced anywhere
			asm("AFP_BreakReady:");
			if ( FALSE )
				asm("jump AFP_BreakReady;");

			// switch on the command
			switch ( AFP_Command )
			{
				// get manufacturer and device codes
				case GET_CODES:
					AFP_Error = GetCodes();
					break;

				// reset
				case RESET:
					AFP_Error = ResetFlash();
					break;

				// write
				case WRITE:
					AFP_Error = WriteData( AFP_Offset, AFP_Count, AFP_Stride, AFP_Buffer );
					break;

				// fill
				case FILL:
					AFP_Error = FillData( AFP_Offset, AFP_Count, AFP_Stride, AFP_Buffer );
					break;

				// erase all
				case ERASE_ALL:
					AFP_Error = EraseFlash();
					break;

				// erase sector
				case ERASE_SECT:
					AFP_Error = EraseBlock( AFP_Sector );
					break;

				// read
				case READ:
					AFP_Error = ReadData( AFP_Offset, AFP_Count, AFP_Stride, AFP_Buffer );
					break;

				// get sector number based on address
				case GET_SECTNUM:
					AFP_Error = GetSectorNumber( AFP_Offset, &AFP_Sector );
					break;

				// get sector number start and end offset
				case GET_SECSTARTEND:
					AFP_Error = GetSectorStartEnd( &AFP_StartOff, &AFP_EndOff, AFP_Sector );

				// no command or unknown command do nothing
				case NO_COMMAND:
				default:
					break;
			}

			// clear the command
			AFP_Command = NO_COMMAND;
		}

		// free the buffer if we were able to allocate one
		if ( AFP_Buffer )
			free( AFP_Buffer );
	}

	// all done
	return 0;
}


//////////////////////////////////////////////////////////////
// ERROR_CODE SetupForFlash()
//
// Perform necessary setup for the processor to talk to the
// flash such as external memory interface registers, etc.
//
//////////////////////////////////////////////////////////////

ERROR_CODE SetupForFlash()
{
	// there is no setup for the 21065L
	return NO_ERR;
}


//////////////////////////////////////////////////////////////
// ERROR_CODE WriteData()
//
// Write a buffer to flash.
//
// Inputs:	unsigned long ulStart - offset in flash to start the writes at
//			long lCount - number of elements to write, in this case bytes
//			long lStride - number of locations to skip between writes
//			int *pnData - pointer to data buffer
//
//////////////////////////////////////////////////////////////

ERROR_CODE WriteData( unsigned long ulStart, long lCount, long lStride, int *pnData )
{
	long i = 0;				// loop counter
	unsigned long ulOffset = ulStart;	// current offset to write
	ERROR_CODE ErrorCode = NO_ERR;		// tells whether we had an error while filling
	int nCompare = 0;					// value that we use to verify flash
	bool bVerifyError = FALSE;			// lets us know if there was a verify error

	// verify writes if the user wants to
	if( AFP_Verify == TRUE )
	{
		// write the buffer up to BUFFER_SIZE items
		for (i = 0; (i < lCount) && ( ErrorCode == NO_ERR ); i++, ulOffset += lStride)
		{
			// unlock the flash, do the write, and wait for completion
			UnlockFlash();
			flash_data = pnData[i];
			WriteFlash( ulOffset, &flash_data );
			ErrorCode = PollToggleBit();
			ReadFlash( ulOffset, &read_data );
			if( ( read_data & 0x000000FF ) != ( pnData[i] & 0x000000FF ) )
			{
				bVerifyError = TRUE;
				break;
			}
		}

		// return appropriate error code if there was a verification error
		if( bVerifyError == TRUE )
			return VERIFY_WRITE;
	}
	else
	{
		// write the buffer up to BUFFER_SIZE items
		for (i = 0; (i < lCount) && ( ErrorCode == NO_ERR ); i++, ulOffset += lStride)
		{
			// unlock the flash, do the write, and wait for completion
			UnlockFlash();
			flash_data = pnData[i];
			WriteFlash( ulOffset, &flash_data );
			ErrorCode = PollToggleBit();
		}
	}

	// ok
	return ErrorCode;
}


//////////////////////////////////////////////////////////////
// ERROR_CODE FillData()
//
// Fill flash with a value.
//
// Inputs:	unsigned long ulStart - offset in flash to start the writes at
//			long lCount - number of elements to write, in this case bytes
//			long lStride - number of locations to skip between writes
//			int *pnData - pointer to data buffer
//
//////////////////////////////////////////////////////////////

ERROR_CODE FillData( unsigned long ulStart, long lCount, long lStride, int *pnData )
{
	long i = 0;				// loop counter
	unsigned long ulOffset = ulStart;	// current offset to write
	ERROR_CODE ErrorCode = NO_ERR;		// tells whether we had an error while filling
	int nCompare = 0;					// value that we use to verify flash
	bool bVerifyError = FALSE;			// lets us know if there was a verify error

	// verify writes if the user wants to
	if( AFP_Verify == TRUE )
	{
		// fill the value
		for (i = 0; (i < lCount) && ( ErrorCode == NO_ERR ); i++, ulOffset += lStride)
		{
			// unlock the flash, do the write, and wait for completion
			UnlockFlash();
			flash_data = pnData[0];
			WriteFlash( ulOffset, &flash_data );
			ErrorCode = PollToggleBit();
			ReadFlash( ulOffset, &read_data );
			if( ( read_data & 0x000000FF ) != ( pnData[0] & 0x000000FF ) )
			{
				bVerifyError = TRUE;
				break;
			}
		}

		// return appropriate error code if there was a verification error
		if( bVerifyError == TRUE )
			return VERIFY_WRITE;
	}
	else
	{
		// fill the value
		for (i = 0; (i < lCount) && ( ErrorCode == NO_ERR ); i++, ulOffset += lStride)
		{
			// unlock the flash, do the write, and wait for completion
			UnlockFlash();
			flash_data = pnData[0];
			WriteFlash( ulOffset, &flash_data );
			ErrorCode = PollToggleBit();
		}
	}

	// ok
	return ErrorCode;
}


//////////////////////////////////////////////////////////////
// ERROR_CODE ReadData()
//
// Read a buffer from flash.
//
// Inputs:	unsigned long ulStart - offset in flash to start the reads at
//			int nCount - number of elements to read, in this case bytes
//			int nStride - number of locations to skip between reads
//			int *pnData - pointer to data buffer to fill
//
//////////////////////////////////////////////////////////////

ERROR_CODE ReadData( unsigned long ulStart, long lCount, long lStride, int *pnData )
{
	long i = 0;				// loop counter
	unsigned long ulOffset = ulStart;	// current offset to write

	// read the buffer up to BUFFER_SIZE items
	for (i = 0; (i < lCount) && (i < BUFFER_SIZE); i++, ulOffset += lStride)
	{
		// do the read
		ReadFlash( ulOffset, &read_data );
		pnData[i] = read_data;
	}

	// ok
	return NO_ERR;
}


//////////////////////////////////////////////////////////////
// ERROR_CODE WriteFlash()
//
// Write a value to an offset in flash.
//
// Inputs:	unsigned long ulOffset - offset into flash
//			int nValue - value to write
//
// Note: Since the flash is mapped using the BMS, the BSO bit
// of SYSCON register is set for enabling the BMS signal
// When the BSO bit is set the DMA accesses only enable the BMS
// signal. Hence the external port DMA 1 is used. 
// For the custom boards where the flash is directly mapped to one 
// of the banks this function needs to be changed accordingly.
//////////////////////////////////////////////////////////////
ERROR_CODE WriteFlash(unsigned long ulOffset, int *flash_data)
{

	// Initialize the SYSCON register BSO bit for enabling the BMS signal.
	
	*pSYSCON |= BSO;
	
	// Initialize the external port 1 registers.
	
	// Initialize the count registers.
	*pC9 = 1;
	*pEC9 = 1;	
	
	// Initialize the Internal Index register with the address of the data
	// need to be written.
	*pII9 = (unsigned int)flash_data;
	*pIM9 = 1;
	
	// Initialize the External Index register with the address of the flash
	// where the data is written.
	*pEI9 = (FLASH_START + ulOffset);
	*pEM9 = 1;
		
	// Enable the external port DMA for transmitting data from internal
	// memory to external memory.
	*pDMAC1 = TRAN | MASTER | DEN;
	
	// Be Idle till the DSP receives the external port interrupt.
	asm("idle;");
	
	// Clear the SYSCON register BSO bit for disabling the BMS signal.
	*pSYSCON &= ~BSO;
	
	// ok
	return NO_ERR;
}



//////////////////////////////////////////////////////////////
// ERROR_CODE ReadFlash()
//
// Read a value from an offset in flash.
//
// Inputs:	unsigned long ulOffset - offset into flash
//			int pnValue - pointer to store value read from flash
//
// Note: Since the flash is mapped using the BMS, the BSO bit
// of SYSCON register is set for enabling the BMS signal
// When the BSO bit is set the DMA accesses only enable the BMS
// signal. Hence the external port DMA 1 is used.
// For the custom boards where the flash is directly mapped to one 
// of the banks this function needs to be changed accordingly.
//////////////////////////////////////////////////////////////
ERROR_CODE ReadFlash(unsigned long ulOffset, int *data)
{
	// Initialize the SYSCON register BSO bit for enabling the BMS signal.
	
	*pSYSCON |= BSO;
	
	// Initialize the external port 1 registers.
	
	// Initialize the count registers.
	*pC9 = 1;
	*pEC9 = 1;	
	
	// Initialize the Internal Index register with the address of the data
	// to where the data is read.
	*pII9 = (unsigned int)data;
	*pIM9 = 1;
	
	// Initialize the External Index register with the address of the flash
	// from where the data is read.
	*pEI9 = (FLASH_START + ulOffset);
	*pEM9 = 1;

	// Enable the external port DMA for transmitting data from external
	// memory to internal memory.		
	*pDMAC1 = MASTER | DEN;
	
	// Be Idle till the DSP receives the external port interrupt.
	asm("idle;");
	
	// Clear the SYSCON register BSO bit for disabling the BMS signal.
	*pSYSCON &= ~BSO;
	
	// ok
	return NO_ERR;
}



//////////////////////////////////////////////////////////////
// ERROR_CODE PollToggleBit()
//
// Polls the toggle bit in the flash to see when the operation
// is complete.
//
//////////////////////////////////////////////////////////////

ERROR_CODE PollToggleBit(void)
{
	int flash_address = 0;
	int temp_data;
	int temp_data1;
	int i;	
	
	ERROR_CODE ErrorCode = POLL_TIMEOUT; // flag to indicate error
		
	ReadFlash(flash_address,&read_data); // read the flash
	
	temp_data = read_data & 0xFF;	// mask valid byte of data from flash
	
	ReadFlash(flash_address,&read_data); // read the flash again
	
	temp_data1 = read_data & 0xFF;	// mask valid byte of data from flash
	
	// check if data toggled if data didn't toggled then erase is finished
	if( temp_data != temp_data1 )	
	{
		// repeat the steps until timeout.
		for( i = 0; i < 0xFFFFFF; i++ )	
		{
			ReadFlash(flash_address,&read_data); // read the flash
	
			temp_data = read_data & 0xFF; // mask valid byte of data from flash
	
			ReadFlash(flash_address,&read_data); // read the flash again
	
			temp_data1 = read_data & 0xFF; // mask valid byte of data from flash

			// check if data toggled 
			if( temp_data == temp_data1 )
			{
				// if data didn't toggled then erase is finished
				ErrorCode = NO_ERR;
				break;
			}
		}
	}
	else
	{
		// return error.
		ErrorCode = NO_ERR;
	} 
	
	// If timed out Reset the flash
	if( ErrorCode == POLL_TIMEOUT) 
		ResetFlash();
		
	// we can return
	return ErrorCode;
}

//////////////////////////////////////////////////////////////
// ERROR_CODE ResetFlash()
//
// Sends a "reset" command to the flash.
//
//////////////////////////////////////////////////////////////

ERROR_CODE ResetFlash()
{
	int flash_address;
	
	// send the reset command to the flash
	flash_address = 0x5555;
	flash_data = 0xaa;
	
	WriteFlash( flash_address, &flash_data);
	
	flash_address = 0x2aaa;
	flash_data = 0x55;
	
	WriteFlash( flash_address, &flash_data);
	
	
	flash_address = 0x5555;
	flash_data = 0xf0;
	
	WriteFlash( flash_address, &flash_data);
	
	// reset should be complete 
	return NO_ERR;
}


//////////////////////////////////////////////////////////////
// ERROR_CODE EraseFlash()
//
// Sends an "erase all" command to the flash.
//
//////////////////////////////////////////////////////////////
ERROR_CODE EraseFlash()
{
	int flash_address;
	ERROR_CODE ErrorCode = NO_ERR;
	
	// Send the commands to erase the flash.
	
	flash_address = 0x5555;
	flash_data = 0xaa;
	
	WriteFlash( flash_address, &flash_data);
	
	flash_address = 0x2aaa;
	flash_data = 0x55;
	
	WriteFlash( flash_address, &flash_data);
	
	
	flash_address = 0x5555;
	flash_data = 0x80;
	
	WriteFlash( flash_address, &flash_data);
	
	
	flash_address = 0x5555;
	flash_data = 0xaa;
	
	WriteFlash( flash_address, &flash_data);
	
	flash_address = 0x2aaa;
	flash_data = 0x55;
	
	WriteFlash( flash_address, &flash_data);
	
	flash_address = 0x5555;
	flash_data = 0x10;
	
	WriteFlash( flash_address, &flash_data); 
	
	// Verify the erase status
	ErrorCode = PollToggleBit();
	
	// we can return
	return ErrorCode;
}


//////////////////////////////////////////////////////////////
// ERROR_CODE EraseBlock()
//
// Sends an "erase block" command to the flash.
//
//////////////////////////////////////////////////////////////

ERROR_CODE EraseBlock( int nBlock )
{
	int nSectorOff = 0x0;
	ERROR_CODE ErrorCode = NO_ERR;	// tells us if there was an error erasing flash
	int flash_address;

	// if the block is invalid just return
	if ( (nBlock < 0) || (nBlock > AFP_NumSectors) )
		return INVALID_BLOCK;

	// send the erase block command to the flash

	flash_address = 0x5555;
	flash_data = 0xaa;
	
	WriteFlash( flash_address, &flash_data);
	
	flash_address = 0x2aaa;
	flash_data = 0x55;
	
	WriteFlash( flash_address, &flash_data);
	
	
	flash_address = 0x5555;
	flash_data = 0x80;
	
	WriteFlash( flash_address, &flash_data);
	
	flash_address = 0x5555;
	flash_data = 0xaa;
	
	WriteFlash( flash_address, &flash_data);
	
	flash_address = 0x2aaa;
	flash_data = 0x55;
	
	WriteFlash( flash_address, &flash_data);

	// the last write has to be at an address in the block
	nSectorOff = (nBlock * 0x10000);

	flash_data = 0x30;
	WriteFlash( nSectorOff, &flash_data);

	// poll until the command has completed
	ErrorCode = PollToggleBit();

	// block erase should be complete
	return ErrorCode;
}


//////////////////////////////////////////////////////////////
// ERROR_CODE UnlockFlash()
//
// Sends an "unlock" command to the flash to allow the flash
// to be programmed.
//
//////////////////////////////////////////////////////////////
ERROR_CODE UnlockFlash()
{
	int flash_address;
	
	// Write commands to unlock the flash.
	flash_address = 0x5555;
	flash_data = 0xaa;
	
	WriteFlash( flash_address, &flash_data);
	
	flash_address = 0x2aaa;
	flash_data = 0x55;
	
	WriteFlash( flash_address, &flash_data);
	
	flash_address = 0x5555;
	flash_data = 0xa0;
	
	WriteFlash( flash_address, &flash_data);	
	
	// ok
	return NO_ERR;
}




//////////////////////////////////////////////////////////////
// ERROR_CODE GetCodes()
//
// Sends an "auto select" command to the flash which will allow
// us to get the manufacturer and device codes.
//
//////////////////////////////////////////////////////////////

ERROR_CODE GetCodes()
{
	int flash_address;
	
	// send the auto select command to the flash

	flash_address = 0x5555;
	flash_data = 0xaa;
	
	WriteFlash( flash_address, &flash_data);
	
	flash_address = 0x2aaa;
	flash_data = 0x55;
	
	WriteFlash( flash_address, &flash_data);
	
	flash_address = 0x5555;
	flash_data = 0x90;
	
	WriteFlash( flash_address, &flash_data);

	
	// now we can read the codes
	
	flash_address = 0x0000;
	ReadFlash( flash_address, &AFP_ManCode );
	
	flash_address = 0x0001;
	ReadFlash( flash_address, &AFP_DevCode );
	
	// we need to issue another command to get the part out
	// of auto select mode so issue a reset which just puts
	// the device back in read mode
	ResetFlash();

	// ok
	return NO_ERR;
}


//////////////////////////////////////////////////////////////
// ERROR_CODE GetSectorNumber()
//
// Gets a sector number based on the offset.
//
//////////////////////////////////////////////////////////////

ERROR_CODE GetSectorNumber( unsigned long ulOffset, int *pnSector )
{
	int nSector = 0;

	// determine the sector
	nSector = ulOffset & 0xffff0000;
	nSector = ulOffset >> 16;
	nSector = nSector & 0x000f;

	// if it is a valid sector, set it
	if ( (nSector >= 0) && (nSector < AFP_NumSectors) )
		*pnSector = nSector;

	// else it is an invalid sector
	else
		return INVALID_SECTOR;

	// ok
	return NO_ERR;
}

//////////////////////////////////////////////////////////////
// ERROR_CODE GetSectorStartEnd()
//
// Gets a sector number based on the offset.
//
// Inputs:	unsigned long *lStartOff - pointer to the start offset
//			unsigned long *lEndOff - pointer to the end offset
//			int nSector - sector number
//
//////////////////////////////////////////////////////////////

ERROR_CODE GetSectorStartEnd( unsigned long *ulStartOff, unsigned long *ulEndOff, int nSector )
{
	// main block
	if( nSector < NUM_SECTORS )
	{
		*ulStartOff = nSector * AFP_SectorSize;
		*ulEndOff = ( (*ulStartOff) + AFP_SectorSize ) - 1;
	}
	else
		return INVALID_SECTOR;


	// ok
	return NO_ERR;
}

//////////////////////////////////////////////////////////////
// void ExternalPort1_DMA_isr(int i)
//
// DMA1 Interrupt Service Rotine
//
// Inputs:	int i(not used).
//
//////////////////////////////////////////////////////////////

void ExternalPort1_DMA_isr(int i)
{
	// Disable the External port 1 DMA in between interrupts.	
	*pDMAC1 = 0;
}


